@@ -115,7 +115,7 @@ class Norm(object):
 
 # {{{ record
 
-class Record(object):
+class RecordWithoutPickling(object):
     """An aggregate of named sub-variables. Assumes that each record sub-type
     will be individually derived from this class.
     """
@@ -138,14 +138,36 @@ class Record(object):
                 fields.add(key)
                 setattr(self, key, value)
 
-    def copy(self, **kwargs):
+    def get_copy_kwargs(self, **kwargs):
         for f in self.__class__.fields:
             if f not in kwargs:
                 try:
                     kwargs[f] = getattr(self, f)
                 except AttributeError:
                     pass
-        return self.__class__(**kwargs)
+        return kwargs
+
+    def copy(self, **kwargs):
+        return self.__class__(**self.get_copy_kwargs(**kwargs))
+
+    def __repr__(self):
+        return "%s(%s)" % (
+                self.__class__.__name__,
+                ", ".join("%s=%r" % (fld, getattr(self, fld))
+                    for fld in self.__class__.fields
+                    if hasattr(self, fld)))
+
+    def register_fields(self, new_fields):
+        try:
+            fields = self.__class__.fields
+        except AttributeError:
+            self.__class__.fields = fields = set()
+
+        fields.update(new_fields)
+
+
+class Record(RecordWithoutPickling):
+    __slots__ = []
 
     def __getstate__(self):
         return dict(
@@ -163,13 +185,6 @@ class Record(object):
             fields.add(key)
             setattr(self, key, value)
 
-    def __repr__(self):
-        return "%s(%s)" % (
-                self.__class__.__name__,
-                ", ".join("%s=%r" % (fld, getattr(self, fld))
-                    for fld in self.__class__.fields
-                    if hasattr(self, fld)))
-
     def __eq__(self, other):
         return (self.__class__ == other.__class__
                 and self.__getstate__() == other.__getstate__())
@@ -354,23 +369,72 @@ def single_valued(iterable, equality_pred=operator.eq):
 
 # {{{ memoization / attribute storage
 
-@my_decorator
-def memoize(func, *args):
-    # by Michele Simionato
-    # http://www.phyast.pitt.edu/~micheles/python/
+def memoize(*args, **kwargs):
+    """Stores previously computed function values in a cache.
 
-    try:
-        return func._memoize_dic[args]
-    except AttributeError:
-        # _memoize_dic doesn't exist yet.
+    Two keyword-only arguments are supported:
 
-        result = func(*args)
-        func._memoize_dic = {args: result}
-        return result
-    except KeyError:
-        result = func(*args)
-        func._memoize_dic[args] = result
-        return result
+    :arg use_kwargs: Allows the caller to use keyword arguments. Defaults to
+        ``False``. Setting this to ``True`` has a non-negligible performance
+        impact.
+    :arg key: A function receiving the same arguments as the decorated function
+        which computes and returns the cache key.
+    """
+
+    use_kw = bool(kwargs.pop('use_kwargs', False))
+
+    if use_kw:
+        def default_key_func(*inner_args, **inner_kwargs):
+            return inner_args, frozenset(inner_kwargs.iteritems())
+    else:
+        default_key_func = None
+
+    key_func = kwargs.pop("key", default_key_func)
+
+    if kwargs:
+        raise TypeError(
+            "memoize received unexpected keyword arguments: %s"
+            % ", ".join(kwargs.keys()))
+
+    if key_func is not None:
+        @my_decorator
+        def _deco(func, *args, **kwargs):
+            # by Michele Simionato
+            # http://www.phyast.pitt.edu/~micheles/python/
+            key = key_func(*args, **kwargs)
+            try:
+                return func._memoize_dic[key]
+            except AttributeError:
+                # _memoize_dic doesn't exist yet.
+                result = func(*args, **kwargs)
+                func._memoize_dic = {key: result}
+                return result
+            except KeyError:
+                result = func(*args, **kwargs)
+                func._memoize_dic[key] = result
+                return result
+    else:
+        @my_decorator
+        def _deco(func, *args):
+            # by Michele Simionato
+            # http://www.phyast.pitt.edu/~micheles/python/
+            try:
+                return func._memoize_dic[args]
+            except AttributeError:
+                # _memoize_dic doesn't exist yet.
+                result = func(*args)
+                func._memoize_dic = {args: result}
+                return result
+            except KeyError:
+                result = func(*args)
+                func._memoize_dic[args] = result
+                return result
+    if not args:
+        return _deco
+    if callable(args[0]) and len(args) == 1:
+        return _deco(args[0])
+    raise TypeError(
+        "memoize received unexpected position arguments: %s" % args)
 
 FunctionValueCache = memoize
 
@@ -416,6 +480,47 @@ def memoize_method(method):
     return new_wrapper
 
 
+def memoize_on_first_arg(function):
+    """Like :func:`memoize_method`, but for functions that take the object
+    to do memoization as first argument.
+
+    Supports cache deletion via ``function_name.clear_cache(self)``.
+
+    .. note::
+        *clear_cache* support requires Python 2.5 or newer.
+    """
+
+    cache_dict_name = intern("_memoize_dic_"
+            + function.__module__ + function.__name__)
+
+    def wrapper(obj, *args, **kwargs):
+        if kwargs:
+            key = (_HasKwargs, frozenset(kwargs.iteritems())) + args
+        else:
+            key = args
+
+        try:
+            return getattr(obj, cache_dict_name)[key]
+        except AttributeError:
+            result = function(obj, *args, **kwargs)
+            setattr(obj, cache_dict_name, {key: result})
+            return result
+        except KeyError:
+            result = function(obj, *args, **kwargs)
+            getattr(obj, cache_dict_name)[key] = result
+            return result
+
+    def clear_cache(obj):
+        delattr(obj, cache_dict_name)
+
+    if sys.version_info >= (2, 5):
+        from functools import update_wrapper
+        new_wrapper = update_wrapper(wrapper, function)
+        new_wrapper.clear_cache = clear_cache
+
+    return new_wrapper
+
+
 def memoize_method_with_uncached(uncached_args=[], uncached_kwargs=set()):
     """Supports cache deletion via ``method_name.clear_cache(self)``.
 
@@ -1396,6 +1501,18 @@ class MovedFunctionDeprecationWrapper:
 
 # {{{ debugging
 
+class StderrToStdout(object):
+    def __enter__(self):
+        import sys
+        self.stderr_backup = sys.stderr
+        sys.stderr = sys.stdout
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        import sys
+        sys.stderr = self.stderr_backup
+        del self.stderr_backup
+
+
 def typedump(val, max_seq=5, special_handlers={}):
     try:
         hdlr = special_handlers[type(val)]
@@ -1598,7 +1715,7 @@ def match_precision(dtype, dtype_to_match):
 
 # {{{ unique name generation
 
-def generate_unique_possibilities(prefix):
+def generate_unique_names(prefix):
     yield prefix
 
     try_num = 0
@@ -1606,6 +1723,9 @@ def generate_unique_possibilities(prefix):
         yield "%s_%d" % (prefix, try_num)
         try_num += 1
 
+generate_unique_possibilities = MovedFunctionDeprecationWrapper(
+        generate_unique_names)
+
 
 class UniqueNameGenerator:
     def __init__(self, existing_names=set(), forced_prefix=""):
@@ -1630,7 +1750,7 @@ class UniqueNameGenerator:
     def __call__(self, based_on="id"):
         based_on = self.forced_prefix + based_on
 
-        for var_name in generate_unique_possibilities(based_on):
+        for var_name in generate_unique_names(based_on):
             if not self.is_name_conflicting(var_name):
                 break
 
